-------------------------------------------------------------------------------
-- Title      : Examples
-- Project    :
-------------------------------------------------------------------------------
-- File       : examples.vhd
-- Author     : Jean-Noel BAZIN  <jnbazin@fl-mee-br-001>
-- Company    :
-- Created    : 2025-03-04
-- Last update: 2025-03-06
-- Platform   :
-- Standard   : VHDL'93/02
-------------------------------------------------------------------------------
-- Description: Brief test bench file to test numeric_std functions behavior
-------------------------------------------------------------------------------
-- Copyright (c) 2025
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2025-03-04  1.0      jnbazin Created
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity examples is
end entity examples;

architecture archi_examples of examples is

    ------------
    -- Resize constant and signals
    ------------

    constant CST_RESIZE_1 : natural := 8;
    constant CST_RESIZE_2 : natural := 8;
    constant CST_RESIZE_3 : natural := 4;

    signal SC_Resize_Operand_1         : signed(CST_RESIZE_1 - 1 downto 0);
    signal SC_Resize_Operand_2         : signed(CST_RESIZE_2 - 1 downto 0);
    signal SC_Resize_Operand_1_resized : signed(CST_RESIZE_1 + CST_RESIZE_2 - 1 downto 0);
    signal SC_Resize_Operand_2_resized : signed(CST_RESIZE_1 + CST_RESIZE_2 - 1 downto 0);
    signal SC_Resize_Result1           : signed(CST_RESIZE_1 + CST_RESIZE_2 - 1 downto 0);
    signal SC_Resize_ResultDownSized   : signed(CST_RESIZE_3 - 1 downto 0);

    ------------
    -- Abs constant and signals
    ------------

    constant CST_ABS_1 : natural := 8;

    signal SC_Abs_Operand_1 : signed(CST_ABS_1 - 1 downto 0);
    signal SC_Abs_result_1  : signed(CST_ABS_1 - 1 downto 0);
    signal SC_Abs_result_2  : signed(CST_ABS_1 downto 0);

    ------------
    -- Add constant and signals
    ------------

    constant CST_ADD_1 : natural := 8;
    constant CST_ADD_2 : natural := CST_ADD_1;
    constant CST_ADD_3 : natural := CST_ADD_1 + 1;

    signal SC_Add_Operand_1 : signed(CST_ADD_1 - 1 downto 0);
    signal SC_Add_Operand_2 : signed(CST_ADD_2 - 1 downto 0);
    signal SC_Add_Result1   : signed(CST_ADD_3 - 1 downto 0);

begin

    ------------
    -- Resize (upsize AND downsize) signal management
    ------------

    SC_Resize_Operand_1 <= to_signed(1, CST_RESIZE_1),
                           to_signed(-1, CST_RESIZE_1)   after 10 ns,
                           to_signed(127, CST_RESIZE_1)  after 20 ns,
                           to_signed(-128, CST_RESIZE_1) after 30 ns,
                           to_signed(127, CST_RESIZE_1)  after 40 ns,
                           to_signed(127, CST_RESIZE_1)  after 50 ns,
                           to_signed(50, CST_RESIZE_1)   after 60 ns,
                           to_signed(-117, CST_RESIZE_1) after 70 ns;

    SC_Resize_Operand_2 <= to_signed(1, CST_RESIZE_2),
                           to_signed(-1, CST_RESIZE_2)   after 10 ns,
                           to_signed(127, CST_RESIZE_2)  after 20 ns,
                           to_signed(-128, CST_RESIZE_2) after 30 ns,
                           to_signed(-128, CST_RESIZE_2) after 40 ns,
                           to_signed(127, CST_RESIZE_2)  after 50 ns,
                           to_signed(50, CST_RESIZE_2)   after 60 ns,
                           to_signed(29, CST_RESIZE_2)   after 70 ns;

    SC_Resize_Operand_1_resized <= resize(SC_Resize_Operand_1, CST_RESIZE_1 + CST_RESIZE_2);
    SC_Resize_Operand_2_resized <= resize(SC_Resize_Operand_2, CST_RESIZE_1 + CST_RESIZE_2);
    SC_Resize_Result1           <= SC_Resize_Operand_1_resized + SC_Resize_Operand_2_resized;
    SC_Resize_ResultDownSized   <= resize(SC_Resize_Result1, CST_RESIZE_3);


    ------------
    -- Abs signal management, with boundary case (-128 on 8 bits)
    ------------

    SC_Abs_Operand_1 <= to_signed(1, CST_RESIZE_1),
                        to_signed(-1, CST_RESIZE_1)   after 10 ns,
                        to_signed(127, CST_RESIZE_1)  after 20 ns,
                        to_signed(-128, CST_RESIZE_1) after 30 ns,
                        to_signed(127, CST_RESIZE_1)  after 40 ns,
                        to_signed(127, CST_RESIZE_1)  after 50 ns,
                        to_signed(50, CST_RESIZE_1)   after 60 ns,
                        to_signed(-117, CST_RESIZE_1) after 70 ns;

    SC_Abs_result_1 <= abs(SC_Abs_Operand_1);
    SC_Abs_result_2 <= abs(resize(SC_Abs_Operand_1, CST_ABS_1 + 1));


    ------------
    -- Add signal management, with boundary cases (-128 on 8 bits)
    ------------
    SC_Add_Operand_1 <= to_signed(1, CST_RESIZE_1),
                        to_signed(-1, CST_RESIZE_1)   after 10 ns,
                        to_signed(127, CST_RESIZE_1)  after 20 ns,
                        to_signed(-128, CST_RESIZE_1) after 30 ns,
                        to_signed(127, CST_RESIZE_1)  after 40 ns,
                        to_signed(127, CST_RESIZE_1)  after 50 ns,
                        to_signed(50, CST_RESIZE_1)   after 60 ns,
                        to_signed(-117, CST_RESIZE_1) after 70 ns;

    SC_Add_Operand_2 <= to_signed(1, CST_RESIZE_2),
                        to_signed(-1, CST_RESIZE_2)   after 10 ns,
                        to_signed(127, CST_RESIZE_2)  after 20 ns,
                        to_signed(-128, CST_RESIZE_2) after 30 ns,
                        to_signed(-128, CST_RESIZE_2) after 40 ns,
                        to_signed(0, CST_RESIZE_2)    after 50 ns,
                        to_signed(50, CST_RESIZE_2)   after 60 ns,
                        to_signed(29, CST_RESIZE_2)   after 70 ns;

    SC_Add_Result1 <= resize(SC_Add_Operand_1, CST_ADD_3) + SC_Add_Operand_2;


end architecture archi_examples;
